/*
 *  maxiMFCC.cpp
 *  mfccs
 *
 *  Created by Chris on 08/03/2011.
 *  Copyright 2011 Goldsmiths Creative Computing. All rights reserved.
 *
 */

#include "maxiMFCC.h"
#include <math.h>
#include <iostream>

using namespace std;

void maxiMFCC::setup(unsigned int numBins, unsigned int numFilters, unsigned int numCoeffs, double minFreq, double maxFreq, unsigned int sampleRate) 
{
	this->numFilters = numFilters;
	this->numCoeffs = numCoeffs;
	this->minFreq = minFreq;
	this->maxFreq = maxFreq;
	this->sampleRate = sampleRate;
	this->numBins = numBins;
	melFilters = NULL;
	melBands = (double*) malloc(sizeof(double) * numFilters);
    dctMatrix = (double**)malloc(sizeof(double*) * numCoeffs);
    //create new matrix
	for(int i=0; i < numFilters; i++) {
		dctMatrix[i] = (double*)malloc(sizeof(double) * numFilters);
	}
	calcMelFilterBank(sampleRate, numBins);
	createDCTCoeffs();
}

void maxiMFCC::mfcc(float* powerSpectrum, double *mfccs) {
	melFilterAndLogSquare(powerSpectrum);
	dct(mfccs);
}


void maxiMFCC::melFilterAndLogSquare(float* powerSpectrum){
//	for(int i=0; i < numBins; i++) {
//		cout << powerSpectrum[i] << ",";
//	}
//	cout << "Bands:\n";
    int frames;
	for (int filter = 0;filter < numFilters;filter++) {
		melBands[filter] = 0.0;
		for (int bin=0;bin<numBins;bin++) {
//			cout << melFilters[filter][bin] << "," << powerSpectrum[bin] << endl;
			melBands[filter] += (melFilters[filter][bin] * powerSpectrum[bin]);
		}
		// log the square
		
		melBands[filter] = melBands[filter] > 0.000001 ? log(melBands[filter] * melBands[filter]) : 0.0;
//		cout << melBands[filter] << ",";
//		if (melBands[filter] > 10000) {
//			cout << "$$$" << endl;
//			for(int z=0; z < numBins; z++) {
//				cout << melFilters[filter][z] << "," << powerSpectrum[z] << endl;
//			}
//			cout << "===============================\n";
//		}
	}
//	cout << endl;

}

void maxiMFCC::dct(double *mfccs){
	for(int i=0; i < numCoeffs; i++) {
		mfccs[i] = 0.0;
	}
	for(int i=0; i < numCoeffs; i++ ) {
		for(int j=0; j < numFilters; j++) {
			mfccs[i] += (dctMatrix[i][j] * melBands[j]);
		}
	}
	for(int i=0; i < numCoeffs; i++) {
		mfccs[i] /= numCoeffs;
	}
}

void maxiMFCC::createDCTCoeffs() {
    double k = 3.14159265358979323846/numFilters;
    double w1 = 1.0/(sqrt(numFilters));
    double w2 = sqrt(2.0/numFilters);
	
	
    //generate dct matrix
    for(int i = 0; i < numCoeffs; i++)
    {
		for(int j = 0; j < numFilters; j++)
		{
			if(i == 0)
				dctMatrix[i][j]= w1 * cos(k * (i+1) * (j + 0.5));
			else
				dctMatrix[i][j] = w2 * cos(k * (i+1) * (j + 0.5));
		}
    }
	
	
}



// implements this formula:
// mel = 2595 log10(Hz/700 + 1)
inline double hzToMel(double hz){
	return 2595.0 * (log10(hz/700.0 + 1.0));
}

// implements this formula
// Hz = 700 (10^(mel/2595) - 1)
inline double melToHz(double mel){
	return 700.0 * (pow(10, mel/2595.0) - 1.0);
}


void maxiMFCC::calcMelFilterBank(double sampleRate, int numBins){
    double mel, dMel, maxMel, minMel, nyquist, binFreq, start, end, thisF, nextF, prevF;
    int numValidBins;
    
    // ignore bins over nyquist
    numValidBins = numBins;
	
    nyquist = sampleRate/2;
    if (maxFreq > nyquist) {
		maxFreq = nyquist;
    }
	
    maxMel = hzToMel(maxFreq);
    minMel = hzToMel(minFreq);
	
    dMel = (maxMel - minMel) / (numFilters + 2 - 1);
    
    double *filtPos = (double*) malloc(sizeof(double) * (numFilters + 2));
	
    // first generate an array of start and end freqs for each triangle
    mel = minMel;
    for (int i=0;i<numFilters + 2;i++) {
		// start of the triangle
		filtPos[i] = melToHz(mel);
		std::cout << "[" << i << "] MFCC: centre is at " <<filtPos[i]<<"hz "<<mel<<" mels" << endl;
		mel += dMel;
    }
    // now generate the coefficients for the mag spectrum
//    double melFilters[numFilters][numValidBins];
	melFilters = (double**) malloc(sizeof(double*) * numFilters);
	for(int i=0; i < numFilters; i++) {
		melFilters[i] = (double*) malloc(sizeof(double) * numValidBins);
	}
	
    for (int filter = 1; filter < numFilters; filter++) {
		for (int bin=0;bin<numValidBins;bin++) {
			// frequency this bin represents
			binFreq = (double) sampleRate / (double) numValidBins * (double) bin;
			thisF = filtPos[filter];
			nextF = filtPos[filter+1];
			prevF = filtPos[filter-1];
			
			//System.out.println("MFCCMYK: filter at "+prevF+","+thisF+","+nextF+" bin at "+binFreq);
			if (binFreq > nextF || binFreq < prevF) {
				// outside this filter
				melFilters[filter][bin] = 0;
				//cout << "MFCCMYK: filter at " <<thisF << " bin at " <<binFreq <<" coeff " <<melFilters[filter][bin] << endl;
			}
			else {
				double height = 2.0 / (nextF - prevF);

				if (binFreq < thisF) {
					// up
					start = prevF;
					end = thisF;
//					melFilters[filter][bin] = (binFreq - start) / (end - start);	  
					melFilters[filter][bin] = (binFreq - start) * (height / (thisF - start));	  
				}
				else {
					// down
					start = thisF;
					end = nextF;
//					melFilters[filter][bin] = 1 - ((binFreq - start) / (end - start));	  
					melFilters[filter][bin] = height + ((binFreq - thisF) * (-height /(nextF - thisF)));	  
				}
				//melFilters[filter][bin] = 1 - (binFreq - start) / (end - start);	  
				cout << "MFCCMYK: filter at " <<thisF << " bin at " <<binFreq <<" coeff " <<melFilters[filter][bin] << endl;
			}
		}
    }
}


maxiMFCC::~maxiMFCC() {
	if (melFilters) {
		for(int i=0; i < numFilters; i++) {
			delete[] melFilters[i];		
		}
		delete[] melFilters;
	}
	delete[] melBands;
}


